@@ -115,6 +115,19 @@ public class Form : ContainerControl
 					}
 				}
 			}
+	[TODO]
+	public bool AllowTransparency
+			{
+				get
+				{
+					// TODO
+					return false;
+				}
+				set
+				{
+					// TODO
+				}
+			}
 	public bool AutoScale
 			{
 				get
@@ -393,6 +406,7 @@ public class Form : ContainerControl
 						return;
 					}
 					mdiParent = value;
+					value.mdiClient.Controls.Add(this);
 				}
 			}
 	public bool MinimizeBox
@@ -658,6 +672,8 @@ public class Form : ContainerControl
 						 cp.Height - ToolkitDrawSize.Height, this);
 				}
 				window.SetTitle(cp.Caption);
+				// Win32 requires this because if the window is maximized, the windows size needs to be set.
+				toolkitWindow = window;
 
 				// Adjust the window hints to match our requirements.
 				SetWindowFlags(window);
@@ -696,7 +712,7 @@ public class Form : ContainerControl
 			{
 				get
 				{
-					return true;
+					return (mdiParent == null);
 				}
 			}
 
@@ -714,10 +730,7 @@ public class Form : ContainerControl
 				{
 					case FormBorderStyle.None:
 					{
-						flags &= ~(ToolkitWindowFlags.Maximize |
-								   ToolkitWindowFlags.ResizeHandles |
-								   ToolkitWindowFlags.Resize |
-								   ToolkitWindowFlags.Border);
+						flags = (ToolkitWindowFlags.ShowInTaskbar);
 					}
 					break;
 
@@ -1094,6 +1107,11 @@ public class Form : ContainerControl
 				}
 			}
 
+	protected void CenterToScreen()
+			{
+				// Nothing to do here -- Not to be used by App developers
+			}
+	
 	// Create a new control collection for this instance.
 	protected override Control.ControlCollection CreateControlsInstance()
 			{
