@@ -101,16 +101,64 @@ class XmlElement : XmlLinkedNode
 			}
 
 	// Get the markup that represents the children of this node.
-	[TODO]
 	public override String InnerXml
 			{
-				get
-				{
-					return base.InnerXml;
-				}
+				get { return base.InnerXml; }
 				set
 				{
-					// TODO
+					// remove the children from this element
+					RemoveContents();
+
+					// bail out now if there's nothing to parse
+					if(value == null || value.Length == 0) { return; }
+
+					// get the owner document
+					XmlDocument doc = FindOwnerQuick();
+
+					// get the name table
+					XmlNameTable nt = doc.NameTable;
+
+					// declare the lang and space
+					String lang; XmlSpace space;
+
+					// create the namespace manager
+					XmlNamespaceManager nm = new XmlNamespaceManager(nt);
+
+					// collect all the ancestor information needed for parsing
+					CollectAncestorInformation(ref nm, out lang, out space);
+
+					// create the parser context
+					XmlParserContext context = new XmlParserContext
+						(nt, nm, lang, space);
+
+					// set the base uri in the parser context
+					context.BaseURI = BaseURI;
+
+					// create the reader
+					XmlTextReader r = new XmlTextReader
+						(value, XmlNodeType.Element, context);
+
+					// move to the first node
+					r.Read();
+
+					// add the child nodes
+					do
+					{
+						// read the next child node
+						XmlNode child = doc.ReadNodeInternal(r);
+
+						// append the child or try to read another
+						if(child != null)
+						{
+							// append the new child node
+							AppendChild(child);
+						}
+						else
+						{
+							r.Read();
+						}
+					}
+					while(r.ReadState == ReadState.Interactive);
 				}
 			}
 
@@ -119,6 +167,10 @@ class XmlElement : XmlLinkedNode
 			{
 				get
 				{
+					// only override the flag if there's content
+					if(FirstChild != null) { isEmpty = false; }
+
+					// return the empty flag
 					return isEmpty;
 				}
 				set
