@@ -29,29 +29,34 @@ using System.Globalization;
 public class XmlTextWriter : XmlWriter
 {
 	// Internal state.
-	internal TextWriter writer;
-	private System.Xml.Formatting formatting;
-	private int indentation;
-	private char indentChar;
-	private bool namespaces;
-	private char quoteChar;
-	private System.Xml.WriteState writeState;
-	private XmlNameTable nameTable;
+	private bool                namespaces;
+	private int                 indentation;
+	private int                 indentLevel;
+	private char                indentChar;
+	private char                quoteChar;
+	private char[]              indentChars;
+	private ElementScope        scope;
+	private Formatting          formatting;
+	private Special             special;
+	private SpecialWriter       specialWriter;
+	private String              nsPrefix;
+	private String              xmlLang;
+	private WriteState          writeState;
+	private XmlNameTable        nameTable;
 	private XmlNamespaceManager namespaceManager;
-	private ElementScope scope;
-	private int indentLevel;
-	private XmlSpace xmlSpace;
-	private String xmlLang;
-	private int pseudoNSNumber;
-	private bool prevWasText;
-	internal bool autoShiftToContent;
+	private XmlSpace            xmlSpace;
+	internal bool               autoShiftToContent;
+	internal TextWriter         writer;
+
 	private enum Special
 	{
-		None  = 0,
-		Lang  = 1,
-		Space = 2
-	}
-	private Special special;
+		None      = 0,
+		Lang      = 1,
+		Space     = 2,
+		Namespace = 3
+
+	}; // enum Special
+
 
 	// Constructors.
 	public XmlTextWriter(Stream w, Encoding encoding)
@@ -80,22 +85,27 @@ public class XmlTextWriter : XmlWriter
 	// Initialize the common writer fields.
 	private void Initialize()
 			{
-				formatting = System.Xml.Formatting.None;
-				indentation = 2;
-				indentChar = ' ';
 				namespaces = true;
+				indentation = 2;
+				indentLevel = 0;
+				indentChar = ' ';
 				quoteChar = '"';
+				indentChars = new char[indentation];
+				scope = null;
+				formatting = System.Xml.Formatting.None;
+				special = Special.None;
+				specialWriter = new SpecialWriter(writer);
+				nsPrefix = null;
+				xmlLang = null;
 				writeState = System.Xml.WriteState.Start;
 				nameTable = new NameTable();
 				namespaceManager = new XmlNamespaceManager(nameTable);
-				scope = null;
-				indentLevel = 0;
 				xmlSpace = XmlSpace.None;
-				xmlLang = null;
-				pseudoNSNumber = 1;
-				prevWasText = false;
-				special = Special.None;
 				autoShiftToContent = false;
+				for(int i = 0; i < indentation; i++)
+				{
+					indentChars[i] = indentChar;
+				}
 			}
 
 	// Push a new element scope.
@@ -107,16 +117,15 @@ public class XmlTextWriter : XmlWriter
 					(localName != null ? nameTable.Add(localName) : null);
 				scope.xmlns = (xmlns != null ? nameTable.Add(xmlns) : null);
 				scope.scopeShown = scopeShown;
+				scope.indentLevel = indentLevel;
 				if(formatting == System.Xml.Formatting.Indented)
 				{
-					scope.indentLevel = indentLevel + indentation;
-				}
-				else
-				{
-					scope.indentLevel = indentLevel;
+					indentLevel++;
 				}
+				scope.indent = false;
 				scope.xmlSpace = xmlSpace;
 				scope.xmlLang = xmlLang;
+				scope.genPrefixCount = 0;
 				namespaceManager.PushScope();
 			}
 
@@ -131,17 +140,24 @@ public class XmlTextWriter : XmlWriter
 			}
 
 	// Add indentation before the current element.
-	private void DoIndent()
+	private void DoIndent(bool useScope)
 			{
-				if(xmlSpace != System.Xml.XmlSpace.Preserve &&
-				   formatting == System.Xml.Formatting.Indented)
+				if(xmlSpace != System.Xml.XmlSpace.Preserve)
 				{
-					int indent = indentLevel;
-					writer.Write(Environment.NewLine);
-					while(indent > 0)
+					if(formatting == System.Xml.Formatting.Indented)
 					{
-						writer.Write(indentChar);
-						--indent;
+						int indent = useScope ? scope.indentLevel : indentLevel;
+						writer.Write(Environment.NewLine);
+						while(indent > 0)
+						{
+							writer.Write(indentChars);
+							--indent;
+						}
+						// flag that perent element needs DoIndent on WriteEndElement
+						if (!useScope && (scope != null))
+						{
+							scope.indent = true;
+						}
 					}
 				}
 			}
@@ -159,9 +175,145 @@ public class XmlTextWriter : XmlWriter
 
 	}; // enum WriteStateFlag
 
-	// Synchronize the output with a particular document area.
-	private void Sync(WriteStateFlag flags)
+
+	// Generate a prefix for the given namespace.
+	private String GeneratePrefix(String ns)
 			{
+				// Create the prefix builder.
+				StringBuilder prefixBuilder = new StringBuilder();
+
+				// Append the start of the prefix.
+				prefixBuilder.Append("d").Append(indentLevel).Append("p");
+
+				// Get the length of the start of the prefix.
+				int len = prefixBuilder.Length;
+
+				// Generate prefixes until an unused one is found.
+				while(true)
+				{
+					// Update the generated prefix count.
+					++scope.genPrefixCount;
+
+					// Build the prefix.
+					prefixBuilder.Append(scope.genPrefixCount);
+
+					// Get the prefix from the builder.
+					String prefix = prefixBuilder.ToString();
+
+					// Get the mapping for the prefix.
+					Object mapping = namespaceManager.LookupNamespace(prefix);
+
+					// Return the prefix if it is unused.
+					if(mapping == null)
+					{
+						// Add the mapping to the namespace manager.
+						namespaceManager.AddNamespace(prefix, ns);
+
+						// Return the prefix.
+						return prefix;
+					}
+
+					// Reset the prefix builder.
+					prefixBuilder.Length = len;
+				}
+			}
+
+	// Handle the end of a special attribute.
+	private void SpecialAttributeEnd()
+			{
+				switch(special)
+				{
+					case Special.Lang:
+					{
+						// Set the xml:lang value.
+						xmlLang = specialWriter.AttributeValue;
+					}
+					break;
+
+					case Special.Space:
+					{
+						// Get the attribute value.
+						String value = specialWriter.AttributeValue;
+
+						// Get the length of the attribute value.
+						int len = value.Length;
+
+						// Set the xml:space value.
+						if(len == 7 && value == "default")
+						{
+							xmlSpace = System.Xml.XmlSpace.Default;
+						}
+						else if(len == 8 && value == "preserve")
+						{
+							xmlSpace = System.Xml.XmlSpace.Preserve;
+						}
+						else
+						{
+							// Reset the attribute value builder.
+							specialWriter.ResetBuilder();
+
+							// Reset the writer.
+							writer = specialWriter.Writer;
+
+							// Reset the special.
+							special = Special.None;
+
+							// Signal that the xml:space value is invalid.
+							throw new ArgumentException(/* TODO */);
+						}
+					}
+					break;
+
+					case Special.Namespace:
+					{
+						// Get the attribute value.
+						String value = specialWriter.AttributeValue;
+
+						// Add the mapping to the namespace manager.
+						namespaceManager.AddNamespace(nsPrefix, value);
+
+						// Reset the namespace prefix.
+						nsPrefix = null;
+					}
+					break;
+				}
+
+				// Reset the attribute value builder.
+				specialWriter.ResetBuilder();
+
+				// Reset the writer.
+				writer = specialWriter.Writer;
+
+				// Reset the special.
+				special = Special.None;
+			}
+
+	// Handle the start of a special attribute.
+	private void SpecialAttributeStart(bool space)
+			{
+				// Set the type of the special attribute.
+				special = (space ? Special.Space : Special.Lang);
+
+				// Set the writer.
+				writer = specialWriter;
+			}
+	private void SpecialAttributeStart(String prefix)
+			{
+				// Set the namespace prefix.
+				nsPrefix = prefix;
+
+				// Set the type of the special attribute.
+				special = Special.Namespace;
+
+				// Set the writer.
+				writer = specialWriter;
+			}
+
+	// Synchronize the output with a particular document area.
+	private void Sync(WriteStateFlag flags, bool indent)
+			{
+				System.Xml.WriteState writeStateOnCalling = writeState;
+
 				// Determine if the current write state is compatible
 				// with the synchronisation flags, and shift to the
 				// requested state if necessary.
@@ -250,13 +402,34 @@ public class XmlTextWriter : XmlWriter
 					}
 					break;
 				}
-				if(writeState == System.Xml.WriteState.Content)
+				if(indent) // do indent if possible
 				{
-					// Record that we wrote some text to a content field.
-					prevWasText = true;
+					bool needIndent;
+
+					if(scope != null)
+					{
+						needIndent = scope.indent;
+					}
+					else
+					{
+						needIndent = false;
+					}
+					if((writeStateOnCalling == System.Xml.WriteState.Prolog) ||
+						(writeStateOnCalling == System.Xml.WriteState.Element) ||
+						(writeStateOnCalling == System.Xml.WriteState.Attribute) ||
+						needIndent)
+					{
+						DoIndent(false);
+					}
 				}
 			}
 
+
+	private void WriteRawData(String value)
+	{
+		writer.Write(value);
+	}
+
 	// Write an xml declaration.
 	private void WriteXmlDeclaration(String text)
 			{
@@ -331,10 +504,6 @@ public class XmlTextWriter : XmlWriter
 						{
 							PopScope();
 						}
-						if(xmlSpace != System.Xml.XmlSpace.Preserve)
-						{
-							writer.WriteLine();
-						}
 					}
 					else if(writeState == System.Xml.WriteState.Element)
 					{
@@ -349,17 +518,18 @@ public class XmlTextWriter : XmlWriter
 					}
 					while(scope != null)
 					{
-						DoIndent();
+						if (scope.indent)
+						{
+							DoIndent(true);
+						}
 						writer.Write("</");
 						writer.Write(scope.localName);
 						writer.Write('>');
 						PopScope();
-						if(xmlSpace != System.Xml.XmlSpace.Preserve)
-						{
-							writer.WriteLine();
-						}
 					}
 
+					// write last newline
+					DoIndent(false);
 					// Flush and close the TextWriter stream.
 					Flush();
 					writer.Close();
@@ -409,7 +579,11 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 		
-				WriteRaw(XmlConvert.ToBase64String(buffer, index, count));
+				// Synchronize to the content or attribute area.
+				Sync(WriteStateFlag.ContentFlag |
+					 WriteStateFlag.AttributeFlag, false);
+
+				WriteRawData(XmlConvert.ToBase64String(buffer, index, count));
 			}
 	
 	// Encode an array as BinHex and write it out as text.
@@ -439,16 +613,13 @@ public class XmlTextWriter : XmlWriter
 				{
 					throw new InvalidOperationException
 						(S._("Xml_InvalidWriteState"));
-				}	
-				char[] hexCode = new char[] {'0','1','2','3','4','5','6','7'
-				,'8','9','A','B','C','D','E','F'};
-				byte currentByte;
-				for(int a = 0; a < count; a++)
-				{
-					currentByte = buffer[index+a];
-					WriteRaw(Convert.ToString(hexCode[currentByte >> 4]));
-					WriteRaw(Convert.ToString(hexCode[currentByte & 0xF]));
 				}
+
+				// Synchronize to the content or attribute area.
+				Sync(WriteStateFlag.ContentFlag |
+					 WriteStateFlag.AttributeFlag, false);
+
+				WriteRawData(XmlConvert.ToHexString(buffer, index, count));	
 			}
 
 	// Write out a CDATA block.
@@ -459,8 +630,9 @@ public class XmlTextWriter : XmlWriter
 					throw new ArgumentException
 						(S._("Xml_InvalidXmlWritten"), "text");
 				}
-				Sync(WriteStateFlag.ContentFlag);
+				Sync(WriteStateFlag.ContentFlag, false);
 				writer.Write("<![CDATA[");
+
 				if(text != null)
 				{
 					writer.Write(text);
@@ -472,7 +644,7 @@ public class XmlTextWriter : XmlWriter
 	public override void WriteCharEntity(char ch)
 			{
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 				writer.Write("&#x{0:X2};", (int)ch);
 			}
 
@@ -497,7 +669,7 @@ public class XmlTextWriter : XmlWriter
 
 				// Synchronize to the content or attribute area.
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 
 				// The buffer must not end in a low surrogate.
 				if(count > 0 &&
@@ -617,17 +789,19 @@ public class XmlTextWriter : XmlWriter
 	// Write a comment.
 	public override void WriteComment(String text)
 			{
-				// Bail out if the comment text contains "-->".
-				if(text != null && text.IndexOf("-->") != -1)
+				// Bail out if the comment text contains "--".
+				if (text !=  null && text.Length > 0)
 				{
-					throw new ArgumentException
-						(S._("Xml_InvalidXmlWritten"), "text");
+					if(text.IndexOf("--") != -1)
+					{
+						throw new ArgumentException
+							(S._("Xml_InvalidXmlWritten"), "text");
+					}
 				}
-
 				// Synchronize to an area that allows comments.
 				Sync(WriteStateFlag.StartFlag |
 				     WriteStateFlag.ContentFlag |
-					 WriteStateFlag.PrologFlag);
+					 WriteStateFlag.PrologFlag, true);
 
 				// Write out the comment.
 				writer.Write("<!--");
@@ -668,6 +842,7 @@ public class XmlTextWriter : XmlWriter
 				}
 
 				// Write out the document type information.
+				DoIndent(false);
 				writer.Write("<!DOCTYPE ");
 				writer.Write(name);
 				writeState = System.Xml.WriteState.Attribute;
@@ -700,7 +875,6 @@ public class XmlTextWriter : XmlWriter
 
 				// Return to the prolog.
 				writeState = System.Xml.WriteState.Prolog;
-				prevWasText = false;
 			}
 
 	// Write the end of an attribute.
@@ -713,10 +887,12 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 
+				// Handle the end of a special attribute, if needed.
+				if(special != Special.None) { SpecialAttributeEnd(); }
+
 				// Terminate the attribute and return to the element state.
 				writer.Write(quoteChar);
 				writeState = System.Xml.WriteState.Element;
-				special = Special.None;
 			}
 
 	// Write the document end information and reset to the start state.
@@ -735,35 +911,25 @@ public class XmlTextWriter : XmlWriter
 					writer.Write(quoteChar);
 					writer.Write(" />");
 					PopScope();
-					if(xmlSpace != System.Xml.XmlSpace.Preserve)
-					{
-						writer.WriteLine();
-					}
 				}
 				else if(writeState == System.Xml.WriteState.Element)
 				{
 					// Terminate the element start.
 					writer.Write(" />");
 					PopScope();
-					if(xmlSpace != System.Xml.XmlSpace.Preserve)
-					{
-						writer.WriteLine();
-					}
 				}
 				while(scope != null)
 				{
-					DoIndent();
+					if (scope.indent)
+					{
+						DoIndent(true);
+					}
 					writer.Write("</");
 					writer.Write(scope.localName);
 					writer.Write('>');
 					PopScope();
-					if(xmlSpace != System.Xml.XmlSpace.Preserve)
-					{
-						writer.WriteLine();
-					}
 				}
 				writeState = System.Xml.WriteState.Start;
-				prevWasText = false;
 			}
 
 	// Write the end of an element and pop the namespace scope.
@@ -778,9 +944,9 @@ public class XmlTextWriter : XmlWriter
 				else if(writeState == System.Xml.WriteState.Content)
 				{
 					// Terminate the element with a full end tag.
-					if(xmlSpace != System.Xml.XmlSpace.Preserve && !prevWasText)
+					if(scope.indent)
 					{
-						DoIndent();
+						DoIndent(true);
 					}
 					writer.Write("</");
 					if(scope.scopeShown && scope.prefix != null && scope.prefix != String.Empty)
@@ -800,7 +966,6 @@ public class XmlTextWriter : XmlWriter
 				// Pop the current scope.
 				PopScope();
 				writeState = System.Xml.WriteState.Content;
-				prevWasText = false;
 			}
 
 	// Write an entity reference.
@@ -812,7 +977,7 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidEntityRef"), "name");
 				}
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 				writer.Write("&{0};", name);
 			}
 
@@ -829,9 +994,9 @@ public class XmlTextWriter : XmlWriter
 				   writeState == System.Xml.WriteState.Content)
 				{
 					// Terminate the element with a full end tag.
-					if(xmlSpace != System.Xml.XmlSpace.Preserve && !prevWasText)
+					if(scope.indent)
 					{
-						DoIndent();
+						DoIndent(true);
 					}
 					writer.Write("</");
 					if(scope.scopeShown && scope.prefix != null && scope.prefix != String.Empty)
@@ -851,7 +1016,6 @@ public class XmlTextWriter : XmlWriter
 				// Pop the current scope.
 				PopScope();
 				writeState = System.Xml.WriteState.Content;
-				prevWasText = false;
 			}
 
 	// Write a name, as long as it is XML-compliant.
@@ -863,7 +1027,7 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidName"), "name");
 				}
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 				writer.Write(name);
 			}
 
@@ -876,7 +1040,7 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidName"), "name");
 				}
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 				writer.Write(name);
 			}
 
@@ -907,16 +1071,16 @@ public class XmlTextWriter : XmlWriter
 				// Synchronize to an area that allows processing instructions.
 				Sync(WriteStateFlag.StartFlag |
 				     WriteStateFlag.PrologFlag |
-				     WriteStateFlag.ContentFlag);
+				     WriteStateFlag.ContentFlag, true);
 
 				// Write out the processing instruction.
 				if(text != null && text.Length != 0)
 				{
-					writer.WriteLine("<?{0} {1}?>", name, text);
+					writer.Write("<?{0} {1}?>", name, text);
 				}
 				else
 				{
-					writer.WriteLine("<?{0}?>",name);
+					writer.Write("<?{0}?>",name);
 				}
 			}
 
@@ -1074,8 +1238,14 @@ public class XmlTextWriter : XmlWriter
 	public override void WriteStartAttribute(String prefix, String localName,
 										     String ns)
 			{
+				// Get the length of the prefix.
+				int prefixLen = (((Object)prefix) == null ? 0 : prefix.Length);
+
+				// Get the length of the namespace.
+				int nsLen = (((Object)ns) == null ? 0 : ns.Length);
+
 				// Validate the parameters.
-				if(!namespaces && (prefix != null || ns != null))
+				if(!namespaces && (prefixLen != 0 || nsLen != 0))
 				{
 					throw new ArgumentException
 						(S._("Xml_NamespacesNotSupported"));
@@ -1084,7 +1254,13 @@ public class XmlTextWriter : XmlWriter
 				// Check the state and output delimiters.
 				if(writeState == System.Xml.WriteState.Attribute)
 				{
+					// Handle the end of a special attribute, if needed.
+					if(special != Special.None) { SpecialAttributeEnd(); }
+
+					// Write the closing quote character for the attribute.
 					writer.Write(quoteChar);
+
+					// Write a space before the start of the attribute.
 					writer.Write(' ');
 				}
 				else if(writeState == System.Xml.WriteState.Element)
@@ -1097,54 +1273,150 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 
+				// Set the special.
+				special = Special.None;
+
 				// Output the name of the attribute, with appropriate prefixes.
-				if(((Object)prefix) != null && prefix != String.Empty &&
-				   ((Object)ns) != null && ns != String.Empty)
+				if(prefixLen != 0)
 				{
-					// We need to associate a prefix with a namespace.
-					String currMapping = LookupPrefix(ns);
-					if(currMapping == prefix)
+					if(prefixLen == 5 && prefix == "xmlns")
 					{
-						// The prefix is already mapped to this URI.
-						if(prefix != scope.prefix)
+						// Ensure the namespace is correct.
+						if(nsLen != 0 &&
+						   (nsLen != 29 ||
+						    ns != "http://www.w3.org/2000/xmlns/"))
 						{
+							throw new ArgumentException(/* TODO */);
+						}
+
+						// Determine if the name starts with (X|x)(M|m)(L|l).
+						bool startsWithXml = localName.ToLower
+							(CultureInfo.InvariantCulture).StartsWith("xml");
+
+						// Ensure that the name is valid.
+						if(startsWithXml)
+						{
+							throw new ArgumentException(/* TODO */);
+						}
+
+						// Handle the start of the special attribute.
+						SpecialAttributeStart(localName);
+					}
+					else if(prefixLen == 3 && prefix == "xml")
+					{
+						// Ensure the namespace is correct.
+						if(nsLen != 0 &&
+						   (nsLen != 36 ||
+						    ns != "http://www.w3.org/XML/1998/namespace"))
+						{
+							throw new ArgumentException(/* TODO */);
+						}
+
+						// Get the length of the local name.
+						int nameLen =
+							(((Object)localName) == null ? 0 : localName.Length);
+
+						// Set the special based on the local name.
+						if(nameLen == 4 && localName == "lang")
+						{
+							// Handle the start of the special attribute.
+							SpecialAttributeStart(false);
+						}
+						else if(nameLen == 5 && localName == "space")
+						{
+							// Handle the start of the special attribute.
+							SpecialAttributeStart(true);
+						}
+					}
+					else if(nsLen != 0)
+					{
+						// Get the current mapping for the namespace.
+						String currMapping = LookupPrefix(ns);
+
+						// Ensure the correct mapping is in scope.
+						if(currMapping != prefix)
+						{
+							// Add the mapping to the namespace manager.
+							namespaceManager.AddNamespace(prefix, ns);
+
+							// Write the namespace declaration.
+							writer.Write("xmlns:");
 							writer.Write(prefix);
-							writer.Write(':');
+							writer.Write('=');
+							writer.Write(quoteChar);
+							WriteQuotedString(ns);
+							writer.Write(quoteChar);
+							writer.Write(' ');
 						}
 					}
 					else
 					{
-						writer.Write("xmlns");
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
-						writer.Write(' ');
+						// Lookup the namespace for the given prefix.
+						ns = namespaceManager.LookupNamespace(prefix);
+
+						// Ensure we have a namespace for the given prefix.
+						if(((Object)ns) == null || ns.Length == 0)
+						{
+							throw new ArgumentException(/* TODO */);
+						}
 					}
+
+					// Write the prefix.
+					writer.Write(prefix);
+					writer.Write(':');
 				}
-				else if(((Object)prefix) != null && prefix != String.Empty)
+				else if(nsLen != 0)
 				{
-					// We were only given a prefix, so output it directly.
-					if(prefix != scope.prefix)
+					if(((Object)localName) != null &&
+					   (localName.Length == 5 && localName == "xmlns"))
 					{
+						// Ensure the namespace is correct.
+						if(nsLen != 29 || ns != "http://www.w3.org/2000/xmlns/")
+						{
+							throw new ArgumentException(/* TODO */);
+						}
+
+						// Handle the start of the special attribute.
+						SpecialAttributeStart(String.Empty);
+					}
+					else if(nsLen == 29 &&
+					        ns == "http://www.w3.org/2000/xmlns/")
+					{
+						throw new ArgumentException(/* TODO */);
+					}
+					else if(nsLen == 36 &&
+					        ns == "http://www.w3.org/XML/1998/namespace")
+					{
+						throw new ArgumentException(/* TODO */);
+					}
+					else
+					{
+						// We were only given a namespace, so find the prefix.
+						prefix = LookupPrefix(ns);
+
+						// Ensure we have a prefix.
+						if(((Object)prefix) == null || prefix.Length == 0)
+						{
+							// Generate a prefix.
+							prefix = GeneratePrefix(ns);
+
+							// Write the namespace declaration for the prefix.
+							writer.Write("xmlns:");
+							writer.Write(prefix);
+							writer.Write('=');
+							writer.Write(quoteChar);
+							WriteQuotedString(ns);
+							writer.Write(quoteChar);
+							writer.Write(' ');
+						}
+
+						// Write the prefix.
 						writer.Write(prefix);
 						writer.Write(':');
 					}
 				}
-				else if(((Object)ns) != null && ns != String.Empty)
-				{
-					// We were only given a namespace, so find the prefix.
-					prefix = LookupPrefix(ns);
-					if(((Object)prefix) == null || prefix.Length == 0)
-					{
-						writer.Write("xmlns");
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
-						writer.Write(' ');
-					}
-				}
+
+				// Write the local name.
 				writer.Write(localName);
 
 				// Output the start of the attribute value.
@@ -1153,149 +1425,8 @@ public class XmlTextWriter : XmlWriter
 
 				// We are now in the attribute state.
 				writeState = System.Xml.WriteState.Attribute;
-
-				// Recognise special attributes.
-				if(prefix == "xml")
-				{
-					if(localName == "lang")
-					{
-						special = Special.Lang;
-					}
-					else if(localName == "space")
-					{
-						special = Special.Space;
-					}
-					else
-					{
-						special = Special.None;
-					}
-				}
-				else
-				{
-					special = Special.None;
-				}
 			}
 
-	// this is sorta a non-spec hack to get namespaces declared from within attributes
-	// to be added to the namespaceManager
-	internal override void WriteStartAttribute(String prefix, String localName,
-										     String ns , String value, bool flagNS)
-			{
-				// heres the hack
-				if((Object)value != null && value != String.Empty && flagNS == true)
-				{
-						// add the namespace and prefix to namespaceManager
-						namespaceManager.AddNamespace(localName, value);
-				}
-				
-				// Validate the parameters.
-				if(!namespaces && (prefix != null || ns != null))
-				{
-					throw new ArgumentException
-						(S._("Xml_NamespacesNotSupported"));
-				}
-
-				// Check the state and output delimiters.
-				if(writeState == System.Xml.WriteState.Attribute)
-				{
-					writer.Write(quoteChar);
-					writer.Write(' ');
-				}
-				else if(writeState == System.Xml.WriteState.Element)
-				{
-					writer.Write(' ');
-				}
-				else if(writeState == System.Xml.WriteState.Closed)
-				{
-					throw new InvalidOperationException
-						(S._("Xml_InvalidWriteState"));
-				}
-
-				// Output the name of the attribute, with appropriate prefixes.
-				if(((Object)prefix) != null && prefix != String.Empty &&
-				   ((Object)ns) != null && ns != String.Empty)
-				{
-					// We need to associate a prefix with a namespace.
-					String currMapping = LookupPrefix(ns);
-					if(currMapping == prefix)
-					{
-						// The prefix is already mapped to this URI.
-						if(prefix != scope.prefix)
-						{
-							writer.Write(prefix);
-							writer.Write(':');
-						}
-					}
-					else
-					{
-						// Create a new pseudo-prefix for the URI.
-						writer.Write("xmlns");
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
-						writer.Write(' ');
-					}
-				}
-				else if(((Object)prefix) != null && prefix != String.Empty)
-				{
-					// We were only given a prefix, so output it directly.
-					if(prefix != scope.prefix)
-					{
-						writer.Write(prefix);
-						writer.Write(':');
-					}
-				}
-				else if(((Object)ns) != null && ns != String.Empty)
-				{
-					// We were only given a namespace, so find the prefix.
-					prefix = LookupPrefix(ns);
-					if(((Object)prefix) == null || prefix.Length == 0)
-					{
-						// Create a new pseudo-prefix for the URI.
-						writer.Write("xmlns");
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
-						writer.Write(' ');
-						writer.Write(prefix);
-						writer.Write(':');
-					}
-				}
-				writer.Write(localName);
-
-				// Output the start of the attribute value.
-				writer.Write('=');
-				writer.Write(quoteChar);
-
-				// We are now in the attribute state.
-				writeState = System.Xml.WriteState.Attribute;
-
-				// Recognise special attributes.
-				if(prefix == "xml")
-				{
-					if(localName == "lang")
-					{
-						special = Special.Lang;
-					}
-					else if(localName == "space")
-					{
-						special = Special.Space;
-					}
-					else
-					{
-						special = Special.None;
-					}
-				}
-				else
-				{
-					special = Special.None;
-				}
-
-			}
-
-	
 	// Write the start of an XML document.
 	public override void WriteStartDocument(bool standalone)
 			{
@@ -1305,13 +1436,13 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 			#if !ECMA_COMPAT
-				writer.WriteLine
+				writer.Write
 					("<?xml version=\"1.0\" encoding=\"{0}\" " +
 					 "standalone=\"{1}\"?>",
 					 writer.Encoding.WebName,
 					 (standalone ? "yes" : "no"));
 			#else
-				writer.WriteLine
+				writer.Write
 					("<?xml version=\"1.0\" standalone=\"{0}\"?>",
 					 (standalone ? "yes" : "no"));
 			#endif
@@ -1327,10 +1458,10 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidWriteState"));
 				}
 			#if !ECMA_COMPAT
-				writer.WriteLine("<?xml version=\"1.0\" encoding=\"{0}\"?>",
+				writer.Write("<?xml version=\"1.0\" encoding=\"{0}\"?>",
 					 			 writer.Encoding.WebName);
 			#else
-				writer.WriteLine("<?xml version=\"1.0\"?>");
+				writer.Write("<?xml version=\"1.0\"?>");
 			#endif
 				writeState = System.Xml.WriteState.Prolog;
 			}
@@ -1340,7 +1471,8 @@ public class XmlTextWriter : XmlWriter
 										   String ns)
 			{
 				// Validate the parameters.
-				if(!namespaces && (prefix != null || ns != null))
+				if(!namespaces &&
+				   (((Object)prefix) != null || ((Object)ns) != null))
 				{
 					throw new ArgumentException
 						(S._("Xml_NamespacesNotSupported"));
@@ -1351,7 +1483,7 @@ public class XmlTextWriter : XmlWriter
 					 WriteStateFlag.AttributeFlag |
 					 WriteStateFlag.ContentFlag |
 					 WriteStateFlag.StartFlag |
-					 WriteStateFlag.PrologFlag);
+					 WriteStateFlag.PrologFlag, true);
 
 				// Get the current scope prefix.
 				String currPrefix;
@@ -1366,13 +1498,9 @@ public class XmlTextWriter : XmlWriter
 
 				// Output the name of the element, with appropriate prefixes.
 				bool scopeShown = false;
-				if(xmlSpace != System.Xml.XmlSpace.Preserve && !prevWasText)
-				{
-					DoIndent();
-				}
 				writer.Write('<');
-				if(((Object)prefix) != null && prefix != String.Empty &&
-				   ((Object)ns) != null && ns != String.Empty)
+				if(((Object)prefix) != null && prefix.Length != 0 &&
+				   ((Object)ns) != null && ns.Length != 0)
 				{
 					// We need to associate a prefix with a namespace.
 					String currMapping = LookupPrefix(ns);
@@ -1390,9 +1518,12 @@ public class XmlTextWriter : XmlWriter
 					else
 					{
 						// Create a new pseudo-prefix for the URI.
+						writer.Write(prefix);
+						writer.Write(':');
 						writer.Write(localName);
 						writer.Write(' ');
-						writer.Write("xmlns");
+						writer.Write("xmlns:");
+						writer.Write(prefix);
 						writer.Write('=');
 						writer.Write(quoteChar);
 						WriteQuotedString(ns);
@@ -1400,7 +1531,7 @@ public class XmlTextWriter : XmlWriter
 						scopeShown = true;
 					}
 				}
-				else if(((Object)prefix) != null && prefix != String.Empty)
+				else if(((Object)prefix) != null && prefix.Length != 0)
 				{
 					// We were only given a prefix, so output it directly.
 					if(prefix != currPrefix)
@@ -1411,24 +1542,24 @@ public class XmlTextWriter : XmlWriter
 					}
 					writer.Write(localName);
 				}
-				else if(((Object)ns) != null && ns != String.Empty)
+				else if(((Object)ns) != null && ns.Length != 0)
 				{
 					// We were only given a namespace, so find the prefix.
 					prefix = LookupPrefix(ns);
-					if((((Object)prefix) == null || prefix.Length == 0) && scope == null)
+					if(((Object)prefix) == null || prefix.Length == 0)
 					{
-						writer.Write(localName);
-						writer.Write(' ');
-						writer.Write("xmlns");
-						writer.Write('=');
-						writer.Write(quoteChar);
-						WriteQuotedString(ns);
-						writer.Write(quoteChar);
-						scopeShown = true;
-					}
-					else if(((Object)prefix) == null || prefix.Length == 0)
-					{
-						if(scope.scopeShown == false || scope.xmlns != ns )
+						if(scope == null)
+						{
+							writer.Write(localName);
+							writer.Write(' ');
+							writer.Write("xmlns");
+							writer.Write('=');
+							writer.Write(quoteChar);
+							WriteQuotedString(ns);
+							writer.Write(quoteChar);
+							scopeShown = true;
+						}
+						else if(scope.scopeShown == false || scope.xmlns != ns)
 						{
 							writer.Write(localName);
 							writer.Write(' ');	
@@ -1444,7 +1575,6 @@ public class XmlTextWriter : XmlWriter
 							writer.Write(localName);
 							scopeShown = true;
 						}
-	
 					}
 					else if(prefix != currPrefix)
 					{
@@ -1468,15 +1598,13 @@ public class XmlTextWriter : XmlWriter
 
 				// Push a new scope record.
 				PushScope(prefix, localName, scopeShown, ns);
-				if(scopeShown)
+				if(scopeShown && ((Object)prefix) != null && prefix.Length != 0)
 				{
-					if(prefix != String.Empty && prefix != null)
-						namespaceManager.AddNamespace(prefix, ns);
+					namespaceManager.AddNamespace(prefix, ns);
 				}
 
 				// We are now in the element state.
 				writeState = System.Xml.WriteState.Element;
-				prevWasText = false;
 			}
 
 	// Write a quoted string.
@@ -1599,7 +1727,7 @@ public class XmlTextWriter : XmlWriter
 				// We must not be in the closed state.
 				if(writeState == System.Xml.WriteState.Closed)
 				{
-					if(text != null && text.Length != 0)
+					if(((Object)text) != null && text.Length != 0)
 					{
 						throw new InvalidOperationException
 							(S._("Xml_InvalidWriteState"));
@@ -1607,36 +1735,9 @@ public class XmlTextWriter : XmlWriter
 					return;
 				}
 
-				// If we are in the attribute state, then check
-				// for the "xml:lang" and "xml:space" attributes.
-				if(writeState == System.Xml.WriteState.Attribute)
-				{
-					if(special == Special.Lang)
-					{
-						xmlLang = text;
-						special = Special.None;
-					}
-					else if(special == Special.Space)
-					{
-						if(text == "default")
-						{
-							xmlSpace = System.Xml.XmlSpace.Default;
-						}
-						else if(text == "preserve")
-						{
-							xmlSpace = System.Xml.XmlSpace.Preserve;
-						}
-						else
-						{
-							xmlSpace = System.Xml.XmlSpace.None;
-						}
-						special = Special.None;
-					}
-				}
-
 				// If we are in the element state, then shift to content.
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 
 				// Bail out if the text is empty.
 				if(((Object)text) == null || text.Length == 0)
@@ -1662,7 +1763,7 @@ public class XmlTextWriter : XmlWriter
 						(S._("Xml_InvalidSurrogate"), "highChar");
 				}
 				Sync(WriteStateFlag.ContentFlag |
-					 WriteStateFlag.AttributeFlag);
+					 WriteStateFlag.AttributeFlag, false);
 				int ch = 0x10000 + ((highChar - 0xD800) << 10) +
 							(lowChar - 0xDC00);
 				writer.Write("&#x{0:X5};", ch);
@@ -1686,7 +1787,7 @@ public class XmlTextWriter : XmlWriter
 				}
 				Sync(WriteStateFlag.ContentFlag |
 					 WriteStateFlag.AttributeFlag |
-					 WriteStateFlag.PrologFlag);
+					 WriteStateFlag.PrologFlag, false);
 				writer.Write(ws);
 			}
 
@@ -1729,6 +1830,10 @@ public class XmlTextWriter : XmlWriter
 				set
 				{
 					indentChar = value;
+					for (int i = 0; i < indentation; i++)
+					{
+						indentChars[i] = indentChar;
+					}
 				}
 			}
 
@@ -1747,6 +1852,11 @@ public class XmlTextWriter : XmlWriter
 							(S._("ArgRange_NonNegative"), "value");
 					}
 					indentation = value;
+					indentChars = new char[indentation];
+					for (int i = 0; i < indentation; i++)
+					{
+						indentChars[i] = indentChar;
+					}
 				}
 			}
 
@@ -1824,6 +1934,8 @@ public class XmlTextWriter : XmlWriter
 		public String xmlLang;			// XmlLang for next higher scope.
 		public ElementScope next;		// The next higher scope.
 		public String xmlns;			// Xmlnamespace in scope.
+		public bool indent;				// flag that surrounding element needs DoIndent on end
+		public int genPrefixCount;      // The count of generated prefixes.
 		public ElementScope(ElementScope next)
 				{
 					this.next = next;
@@ -1831,6 +1943,107 @@ public class XmlTextWriter : XmlWriter
 
 	}; // class ElementScope
 
+	// Class that keeps track of special attribute values, on output, as needed.
+	private sealed class SpecialWriter : TextWriter
+	{
+		// Internal state.
+		private StringBuilder builder;
+		private TextWriter    writer;
+
+
+		// Constructor.
+		public SpecialWriter(TextWriter writer)
+				: base()
+				{
+					this.builder = new StringBuilder();
+					this.writer = writer;
+				}
+
+
+		// Get the attribute value.
+		public String AttributeValue
+				{
+					get { return builder.ToString(); }
+				}
+
+		// Get the attribute value builder.
+		public StringBuilder Builder
+				{
+					get { return builder; }
+				}
+
+		// Get the encoding for this writer.
+		public override Encoding Encoding
+				{
+					get
+					{
+						if(writer == null) { return null; }
+						return writer.Encoding;
+					}
+				}
+
+		// Get the text writer.
+		public TextWriter Writer
+				{
+					get { return writer; }
+				}
+
+
+		// Close this writer.
+		public override void Close()
+				{
+					Dispose(false);
+				}
+
+		// Dispose of this writer.
+		protected override void Dispose(bool disposing)
+				{
+					if(writer != null)
+					{
+						if(disposing)
+						{
+							((IDisposable)writer).Dispose();
+						}
+						else
+						{
+							writer.Close();
+						}
+						builder = null;
+						writer = null;
+					}
+				}
+
+		// Reset the attribute value builder.
+		public void ResetBuilder()
+				{
+					builder.Length = 0;
+				}
+
+		// Write values to this writer.
+		public override void Write(String value)
+				{
+					if(((Object)value) != null)
+					{
+						writer.Write(value);
+						builder.Append(value);
+					}
+				}
+		public override void Write(char[] buffer, int index, int count)
+				{
+					if(count > 0)
+					{
+						writer.Write(buffer, index, count);
+						builder.Append(buffer, index, count);
+					}
+				}
+		public override void Write(char ch)
+				{
+					writer.Write(ch);
+					builder.Append(ch);
+				}
+
+	}; // class SpecialWriter
+
 }; // class XmlTextWriter
 
 }; // namespace System.Xml
