@@ -27,7 +27,7 @@ using System.IO;
 public interface IQDataStreamable
 {
 	// Read the state of this object from a data stream.
-	void Read(QDataStream stream);
+	Object Read(QDataStream stream);
 
 	// Write this object's state to a data stream.
 	void Write(QDataStream stream);
