@@ -555,7 +555,7 @@ public class TopLevelWindow : InputOutputWidget
 					Xlib.XReconfigureWMWindow
 							(display, handle,
 							 Screen.ScreenNumber,
-						     (uint)(ConfigureWindowMask.CWStackMode),
+						     (uint)mask,
 							 ref changes);
 				}
 			}
@@ -776,7 +776,7 @@ public class TopLevelWindow : InputOutputWidget
 				(IntPtr display, XWindow handle, bool maximize)
 			{
 				XEvent xevent = new XEvent();
-				xevent.xany.type = (int)(EventType.ClientMessage);
+				xevent.xany.type = (int)(Xsharp.Events.EventType.ClientMessage);
 				xevent.xany.window = handle;
 				xevent.xclient.message_type = Xlib.XInternAtom
 					(display, "_NET_WM_STATE", XBool.False);
@@ -1488,6 +1488,16 @@ public class TopLevelWindow : InputOutputWidget
 				}
 			}
 
+	/// <summary>
+	/// <para>Method that is called when a custom "BeginInvoke" message comes in
+	/// changes.</para>
+	/// </summary>
+	protected virtual void OnBeginInvokeMessage(IntPtr i_gch)
+			{
+				// Nothing to do in this base class.
+			}
+
+
 	/// <summary>
 	/// <para>Method that is called when the window's iconic state
 	/// changes.</para>
@@ -1795,27 +1805,24 @@ public class TopLevelWindow : InputOutputWidget
 				InputOnlyWidget io;
 				Xlib.Xlong[] data;
 
-				switch((EventType)(xevent.xany.type__))
+				switch((Xsharp.Events.EventType)(xevent.xany.type__))
 				{
-					case EventType.ClientMessage:
-					{
+					case Xsharp.Events.EventType.ClientMessage:
+						{
 						// Handle messages from the window manager.
 						if(xevent.xclient.message_type == dpy.wmProtocols)
 						{
-							if(xevent.xclient.l(0) ==
-									(int)(dpy.wmDeleteWindow))
+							if(xevent.xclient.l(0) == (int)(dpy.wmDeleteWindow))
 							{
 								// User wants the window to close.
 								Close();
 							}
-							else if(xevent.xclient.l(0) ==
-										(int)(dpy.wmTakeFocus))
+							else if(xevent.xclient.l(0) == (int)(dpy.wmTakeFocus))
 							{
 								// We were given the primary input focus.
 								PrimaryFocusIn();
 							}
-							else if(xevent.xclient.l(0) ==
-										(int)(dpy.wmContextHelp))
+							else if(xevent.xclient.l(0) == (int)(dpy.wmContextHelp))
 							{
 								// The user pressed the "help" button.
 								OnHelp();
@@ -1828,8 +1835,7 @@ public class TopLevelWindow : InputOutputWidget
 								try
 								{
 									IntPtr display = dpy.Lock();
-									xevent.xany.window =
-										screen.RootWindow.GetWidgetHandle();
+									xevent.xany.window = screen.RootWindow.GetWidgetHandle();
 									Xlib.XSendEvent
 										(display, xevent.xany.window,
 										 XBool.False,
@@ -1842,10 +1848,14 @@ public class TopLevelWindow : InputOutputWidget
 								}
 							}
 						}
+						else if(xevent.xclient.message_type == dpy.internalBeginInvoke)
+						{
+							OnBeginInvokeMessage((IntPtr)xevent.xclient.l(0));
+						}
 					}
 					break;
 
-					case EventType.PropertyNotify:
+				case Xsharp.Events.EventType.PropertyNotify:
 					{
 						// Handle a property change notification.
 						if(xevent.xproperty.atom == dpy.wmState)
@@ -1920,17 +1930,21 @@ public class TopLevelWindow : InputOutputWidget
 								}
 							}
 						}
+						else if(xevent.xclient.message_type == dpy.internalBeginInvoke)
+						{
+							OnBeginInvokeMessage((IntPtr)xevent.xclient.l(0));
+						}
 					}
 					break;
 
-					case EventType.FocusIn:
+				case Xsharp.Events.EventType.FocusIn:
 					{
 						// This window has received the focus.
 						PrimaryFocusIn();
 					}
 					break;
 
-					case EventType.FocusOut:
+				case Xsharp.Events.EventType.FocusOut:
 					{
 						// This window has lost the focus.
 						if(hasPrimaryFocus)
@@ -1945,7 +1959,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.KeyPress:
+				case Xsharp.Events.EventType.KeyPress:
 					{
 						// Convert the event into a symbol and a string.
 						if(keyBuffer == IntPtr.Zero)
@@ -2021,7 +2035,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.KeyRelease:
+				case Xsharp.Events.EventType.KeyRelease:
 					{
 						// Convert the event into a symbol and a string.
 						if(keyBuffer == IntPtr.Zero)
@@ -2055,7 +2069,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.ConfigureNotify:
+				case Xsharp.Events.EventType.ConfigureNotify:
 					{
 						// The window manager may have caused us to move/resize.
 						if(xevent.xconfigure.window != xevent.window)
@@ -2109,7 +2123,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.ReparentNotify:
+				case Xsharp.Events.EventType.ReparentNotify:
 					{
 						// We may have been reparented by the window manager.
 						if(xevent.xreparent.window != (XWindow)handle)
@@ -2134,7 +2148,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.MapNotify:
+				case Xsharp.Events.EventType.MapNotify:
 					{
 						// The window manager mapped us to the screen.
 						if(Parent is CaptionWidget)
@@ -2154,7 +2168,7 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
-					case EventType.UnmapNotify:
+				case Xsharp.Events.EventType.UnmapNotify:
 					{
 						// We were unmapped from the screen.  If "mapped"
 						// is true, then we are being iconified by the window
