@@ -388,7 +388,7 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 	// Determine if this sorted list contains a particular value.
 	public virtual bool ContainsValue(Object value)
 			{
-				return (Array.IndexOf(keys, value, 0, count) != -1);
+				return (Array.IndexOf(values, value, 0, count) != -1);
 			}
 
 	// Get a value by index.
@@ -483,7 +483,7 @@ public class SortedList : IDictionary, ICollection, IEnumerable, ICloneable
 	// Trim the capacity of this sorted list to its actual size.
 	public virtual void TrimToSize()
 			{
-				if(count != keys.Length)
+				if(count != keys.Length && keys.Length > DefaultCapacity)
 				{
 					Object[] newKeys = new Object [count];
 					Object[] newValues = new Object [count];
